# Keycloak Configuration for groceryAggregator

This repository/directory was created as part of my thesis and contains configuration files and Docker settings for Keycloak, specifically adapted for the groceryAggregator-fe (frontend) and groceryAggregator-be (backend) applications. The goal is to provide you with a simple and efficient way to set up authentication and authorization for this web application. This application focuses on aggregating information from various online grocery stores in the Czech Republic. Thanks to automation through Docker and Docker Compose, the entire setup process is greatly simplified, allowing for immediate testing possibilities while maintaining a high level of security (delegated to a third party - in this case, Keycloak).

## Login Information

For testing and educational reasons, the following access details are freely available.

After successfully launching Keycloak, you can access the administration console using the following login credentials:

- **Username**: admin
- **Password**: admin

Also, a user is pre-set for testing integration:

- **Username**: myuser
- **Password**: mypass1


## Getting Started

Follow these steps to easily launch Keycloak and integrate it with the groceryAggregator applications.

### Prerequisites

Before starting, make sure you have installed Docker and Docker Compose (in the latest versions, they are usually included with Docker). These tools are essential for automatic setup and launching of Keycloak. If you do not have Docker yet, visit the [official Docker page](https://www.docker.com/get-started) and follow the instructions for installation.

### Launching Keycloak

1. Open a terminal or command line.
2. Navigate to the directory of this repository.
3. Start the containers using Docker Compose:

    ```
    docker-compose up
    ```

    Alternatively, you can use `start.cmd` (for Windows) or `start.sh` (for Unix/Linux) for easy launching:

    ```
    ./start.cmd
    ```

    or

    ```
    ./start.sh
    ```

This downloads the necessary Docker image (Keycloak), creates and starts the container, and automatically imports the preconfiguration files.

## Technologies Used

- [Docker](https://www.docker.com/) - Software containerization.
- [Docker Compose](https://docs.docker.com/compose/) - A tool for defining and running multi-container Docker applications.
- [Keycloak](https://www.keycloak.org/) - Open Source Identity and Access Management used for authentication and authorization.
