package cz.gottlieb.groceryAggregator.configs.restTemplate;

import cz.gottlieb.groceryAggregator.utils.Constants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpRequest;
import org.springframework.http.client.ClientHttpRequestExecution;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.SessionScope;

import java.io.IOException;
import java.util.List;
import java.util.Objects;

@Slf4j
@SessionScope
@Component
public class BillaInterceptor implements ClientHttpRequestInterceptor {

    private String xsrfToken = "";

    @Override
    public ClientHttpResponse intercept(HttpRequest request, byte[] body, ClientHttpRequestExecution execution) throws IOException {
        log.info(">>> intercept");

        setHeaders(request.getHeaders());

        ClientHttpResponse response = execution.execute(request, body);

        List<String> setterCookies = response.getHeaders().get(HttpHeaders.SET_COOKIE);
        parseCsrfToken(setterCookies);

        log.info("<<< intercept");
        return response;
    }

    private void setHeaders(HttpHeaders headers) {
        headers.set("Accept", "application/json");
        headers.set("Accept-Encoding", "identity"); //identity, br // gzip, deflate
        headers.set("Accept-Language", "cs-CZ,cs;q=0.9,en;q=0.8");
        headers.set("Referer", "https://shop.billa.cz/kosik");
        headers.set("User-Agent", Constants.userAgent);

        if (!xsrfToken.isBlank()) {
            headers.set("X-XSRF-TOKEN", xsrfToken);
        }
    }

    private void parseCsrfToken(List<String> setterCookies) {
        if (setterCookies != null) {
            xsrfToken = setterCookies.stream()
                    .filter(Objects::nonNull)
                    .filter(c -> c.startsWith("XSRF-TOKEN="))
                    .map(c -> c.replace("XSRF-TOKEN=", ""))
                    .map(c -> c.substring(0, c.indexOf(";")))
                    .filter(s -> !s.isEmpty())
                    .findFirst()
                    .orElse(xsrfToken);
        }
    }
}
