package cz.gottlieb.groceryAggregator.configs.security;

import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.CorsConfigurationSource;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;

import java.util.List;

@Slf4j
//@EnableMethodSecurity
@EnableGlobalMethodSecurity(prePostEnabled = true)
@EnableWebSecurity
public class SecurityConfig {

    @Bean
    public SecurityFilterChain filterChain(HttpSecurity http,
                                           Jwt2AuthenticationConverter authenticationConverter,
                                           ServerProperties serverProperties) throws Exception {

        // Enable and configure CORS (by Bean Injection...)
        http.cors();

        http.csrf().ignoringAntMatchers("/technical/**");

        // Enable OAuth2 with custom authorities mapping
        http.oauth2ResourceServer().jwt().jwtAuthenticationConverter(authenticationConverter);

        setAuthorizeHttpRequests(http);

        // If SSL enabled, disable http (https only)
        if (serverProperties.getSsl() != null && serverProperties.getSsl().isEnabled()) {
            http.requiresChannel().anyRequest().requiresSecure();
        } else {
            http.requiresChannel().anyRequest().requiresInsecure();
        }

        return http.build();
    }

    @Bean
    public CorsConfigurationSource corsConfigurationSource() {
        CorsConfiguration config = new CorsConfiguration();

        config.setAllowCredentials(true);

        config.setAllowedOriginPatterns(List.of(
                "http://localhost:[*]", "https://localhost:[*]",
                "http://aggr-fe.azurewebsites.net", "https://aggr-fe.azurewebsites.net"
        ));

        config.setAllowedMethods(List.of("GET", "POST", "PUT", "DELETE", "OPTIONS", "PATCH"));

        config.setAllowedHeaders(List.of("*"));

        final UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/**", config);

        return source;
    }

    // Not used due to the use of class and method annotations
    private void setAuthorizeHttpRequests(HttpSecurity http) throws Exception {
        // Enable anonymous
//        http.anonymous(); // check ROLE_ANONYMOUS

        // Route security: authenticated to all routes but actuator and Swagger-UI
        http.authorizeHttpRequests(authzRequest -> authzRequest
                        .antMatchers(
                                "/technical/healthCheck",
                                "/technical/run/watchdog",
//                        "/v3/api-docs",
//                        "/v3/api-docs/**",
                                "/swagger-ui/**",
                                "/swagger-ui.html"
                        ).permitAll()

                        // Set by class and methods annotations
//                      .antMatchers("/user/**").hasAuthority("ROLE_user")
//                      .antMatchers("/admin/**").hasAuthority("ROLE_admin")

//                      .antMatchers(GET, "/technical/healthCheck").hasAuthority("SCOPE_read", "ROLE_user")
//                      .antMatchers(GET, "/technical/run/watchdog").hasRole("technical")

                        .anyRequest().authenticated() //.denyAll() //.permitAll()
        );
    }

}
