package cz.gottlieb.groceryAggregator.configs.security;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationToken;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

@Configuration
public class SecurityConverter {

    public SecurityConverter() {
    }

    @Bean
    public Jwt2AuthenticationConverter authenticationConverter(Jwt2AuthoritiesConverter authoritiesConverter) {
        return jwt -> new JwtAuthenticationToken(jwt, authoritiesConverter.convert(jwt));
    }

    @SuppressWarnings("unchecked")
    @Bean
    public Jwt2AuthoritiesConverter authoritiesConverter() {
        // Roles are taken from realm_access.roles & resource_access.account.roles
        // Scopes are taken from scope
        return jwt -> {
            final var realmAccess = (Map<String, Object>) jwt.getClaims().getOrDefault("realm_access", Map.of());
            final var realmRoles = (Collection<String>) realmAccess.getOrDefault("roles", List.of());

            final var resourceAccess = (Map<String, Object>) jwt.getClaims().getOrDefault("resource_access", Map.of());
            final var clientAccess = (Map<String, Object>) resourceAccess.getOrDefault("account", Map.of());
            final var clientRoles = (List<String>) clientAccess.getOrDefault("roles", List.of());

            List<String> rolesWithPrefix = Stream.concat(realmRoles.stream(), clientRoles.stream())
                    .map(s -> "ROLE_".concat(s)).toList();

            final var scopesString = (String) jwt.getClaims().getOrDefault("scope", "");
            List<String> scopesWithPrefix = Arrays.stream(scopesString.split("\\s+")).map(s -> "SCOPE_".concat(s)).toList();

            final var mergedAll = Stream.concat(rolesWithPrefix.stream(), scopesWithPrefix.stream()).toList();
            return mergedAll.stream().map(SimpleGrantedAuthority::new).toList();
        };
    }
}
