package cz.gottlieb.groceryAggregator.controllers;

import cz.gottlieb.groceryAggregator.persistence.model.Company;
import cz.gottlieb.groceryAggregator.persistence.repository.CompanyRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Comparator;
import java.util.List;

@Slf4j
@Transactional
@RestController
@PreAuthorize("hasAnyRole('user', 'admin') and hasAuthority('SCOPE_read')")
@RequestMapping("/company")
public class CompanyController {

    private final CompanyRepository companyRepository;

    public CompanyController(CompanyRepository companyRepository) {
        this.companyRepository = companyRepository;
    }

    @GetMapping
    public ResponseEntity<List<Company>> getAllCompaniesCredentials() {
        log.info(">>> getAllCompaniesCredentials");

        List<Company> companies = companyRepository.findAll().stream()
                .sorted(Comparator.comparing(Company::getId)).toList();

        return new ResponseEntity<>(companies, HttpStatus.OK);
    }
}
