package cz.gottlieb.groceryAggregator.controllers;

import cz.gottlieb.groceryAggregator.annotation.CurrentUser;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

@Slf4j
@Transactional
@RestController
@PreAuthorize("hasAnyRole('user', 'admin')")
@RequestMapping("/user")
public class UserContoller {

    @Value("${server.servlet.session.cookie.name}")
    private String cookieName;

    public UserContoller() {
    }

    @PostMapping("/logout")
    public ResponseEntity<?> logout(@CurrentUser User user,
                                    HttpServletRequest request,
                                    HttpServletResponse response) {

        boolean isSecure = false;
        String contextPath = null;

        if (request != null) {
            HttpSession session = request.getSession(false);
            if (session != null) {
                session.invalidate();
            }
            isSecure = request.isSecure();
            contextPath = request.getContextPath();
        }

        SecurityContext context = SecurityContextHolder.getContext();
        context.setAuthentication(null);
        SecurityContextHolder.clearContext();

        if (response != null) {
            Cookie cookie = new Cookie(cookieName, null);
            String cookiePath = StringUtils.hasText(contextPath) ? contextPath : "/";
            cookie.setPath(cookiePath);
            cookie.setMaxAge(0);
            cookie.setSecure(isSecure);
            response.addCookie(cookie);
        }

        return new ResponseEntity<>(true, HttpStatus.OK);
    }
}
