package cz.gottlieb.groceryAggregator.dto.mapper;

import cz.gottlieb.groceryAggregator.dto.billa.BillaCredentialDto;
import cz.gottlieb.groceryAggregator.dto.billa.BillaItemRequestDto;
import cz.gottlieb.groceryAggregator.dto.billa.BillaItemResponseDto;
import cz.gottlieb.groceryAggregator.dto.billa.BillaProductResponseDto;
import cz.gottlieb.groceryAggregator.persistence.model.Company;
import cz.gottlieb.groceryAggregator.persistence.model.GroceryCredential;
import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingListItem;
import cz.gottlieb.groceryAggregator.utils.Constants;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

import static cz.gottlieb.groceryAggregator.configs.connectors.impl.BillaConfigBaseImpl.serverBaseUrl;

public class DtoBillaMapper {

    //<editor-fold desc="FE --(add|modify)--> Cart">
    public static BillaItemRequestDto mapToBillaCartItemPostDto(ShoppingListItem shoppingListItem) {
        BillaItemRequestDto billaDto = new BillaItemRequestDto();

        billaDto.setSku(shoppingListItem.getProduct().getOriginSku());
        billaDto.setQuantity(shoppingListItem.getQuantity());

        return billaDto;
    }

    public static BillaItemRequestDto mapToBillaCartItemPutDto(ShoppingListItem shoppingListItem) {
        BillaItemRequestDto billaDto = new BillaItemRequestDto();

        billaDto.setSku(shoppingListItem.getProduct().getOriginSku());
        billaDto.setQuantity(shoppingListItem.getQuantity());

        return billaDto;
    }
    //</editor-fold>

    //<editor-fold desc="Cart --> FE">
    public static List<ShoppingListItem> mapToShoppingListItem(List<BillaItemResponseDto> billaItemResponseDtos, Company company) {
        return billaItemResponseDtos.stream()
                .map(billaItemResponseDto -> mapToShoppingListItem(billaItemResponseDto, company))
                .collect(Collectors.toList());
    }

    public static ShoppingListItem mapToShoppingListItem(BillaItemResponseDto billaItemResDto, Company company) {

        Product product = mapToProduct(billaItemResDto.getProduct(), company);

        ShoppingListItem shoppingListItem = new ShoppingListItem();

        shoppingListItem.setOriginItemId(billaItemResDto.getLineItemId());
        shoppingListItem.setQuantity(billaItemResDto.getQuantity());
        shoppingListItem.setPrice(new BigDecimal(billaItemResDto.getPrice()));
        shoppingListItem.setProduct(product);

        return shoppingListItem;
    }
    //</editor-fold>

    //<editor-fold desc="Billa products --> Model">
    public static List<Product> mapToProduct(List<BillaProductResponseDto> billaProductResponseDtos, Company company) {
        return billaProductResponseDtos.stream()
                .map(billaDto -> mapToProduct(billaDto, company))
                .collect(Collectors.toList());
    }

    public static Product mapToProduct(BillaProductResponseDto billaDto, Company company) {
        Product product = new Product();

        product.setCompany(company);

        product.setName(billaDto.getName());
        product.setOriginProductId(billaDto.getProductId());
        product.setOriginSku(billaDto.getSku());

        product.setUnit(billaDto.getVolumeLabelShort());
        product.setUnitStandardized(billaDto.getPrice().getBaseUnitShort());
        product.setAmount(billaDto.getAmount() + " " + billaDto.getVolumeLabelShort());
        product.setMaxAvailableAmount(-1); // not supported by Billa

        product.setPriceFull((new BigDecimal(billaDto.getPrice().getRegular().getValue()))
                .divide(BigDecimal.valueOf(100))); // Billa represents the price in hellers
        product.setPriceCurrency(Constants.CZK); // Billa supports CZK only

        product.setPricePerUnitFull((new BigDecimal(billaDto.getPrice().getRegular().getPerStandardizedQuantity()))
                .divide(BigDecimal.valueOf(100))); // Billa represents the price in hellers
        product.setPricePerUnitCurrency(Constants.CZK); // Billa supports CZK only

        product.setBaseUrl(serverBaseUrl + "/produkt/" + billaDto.getSlug());
        product.setImgPath(billaDto.getImages().stream().findFirst().orElse(""));
        product.setDescription(billaDto.getProductMarketing());
        product.setUpdateAt(LocalDateTime.now());

        return product;
    }
    //</editor-fold>

    //<editor-fold desc="Credential model --> Billa">
    public static BillaCredentialDto mapToBillaCredentialDto(GroceryCredential credential) {
        BillaCredentialDto billaCredentialDto = new BillaCredentialDto();

        billaCredentialDto.setPassword(credential.getPassword());
        billaCredentialDto.setEmail(credential.getUsername());

        return billaCredentialDto;
    }
    //</editor-fold>
}
