package cz.gottlieb.groceryAggregator.dto.mapper;

import cz.gottlieb.groceryAggregator.dto.fe.*;
import cz.gottlieb.groceryAggregator.persistence.model.*;
import cz.gottlieb.groceryAggregator.utils.ShoppingListEnum;

import java.util.List;
import java.util.stream.Collectors;

public class DtoFeMapper {

    //<editor-fold desc="FE --> WatchDog model">
    public static List<FeWatchDogDto> mapToFeWatchDogDto(List<WatchDog> watchDogs) {
        return watchDogs.stream()
                .map(DtoFeMapper::mapToFeWatchDogDto)
                .collect(Collectors.toList());
    }

    public static FeWatchDogDto mapToFeWatchDogDto(WatchDog watchDog) {
        FeWatchDogDto feWatchDogDto = new FeWatchDogDto();

        feWatchDogDto.setId(watchDog.getId());
        feWatchDogDto.setCompany(watchDog.getProduct().getCompany().getName().name());

        feWatchDogDto.setOriginSku(watchDog.getProduct().getOriginSku());
        feWatchDogDto.setProductName(watchDog.getProduct().getName());
        feWatchDogDto.setProductUrl(watchDog.getProduct().getBaseUrl());

        feWatchDogDto.setPriceAlarm(watchDog.getPriceAlarm());

        return feWatchDogDto;
    }

    public static WatchDog mapToWatchDog(FeWatchDogDto feWatchDogDto, Product product) {
        WatchDog watchDog = new WatchDog();

        watchDog.setId(feWatchDogDto.getId());
        watchDog.setPriceAlarm(feWatchDogDto.getPriceAlarm());
        watchDog.setProduct(product);

        return watchDog;
    }
    //</editor-fold>

    //<editor-fold desc="FE --> ShoppingList model">
    public static List<FeShoppingListResponseDto> mapToFeShoppingListDto(List<ShoppingList> shoppingLists) {
        return shoppingLists.stream()
                .map(DtoFeMapper::mapToFeShoppingListDto)
                .collect(Collectors.toList());
    }

    public static FeShoppingListResponseDto mapToFeShoppingListDto(ShoppingList shoppingList) {
        FeShoppingListResponseDto feShoppingListDto = new FeShoppingListResponseDto();

        feShoppingListDto.setId(shoppingList.getId());
        feShoppingListDto.setName(shoppingList.getName());
        feShoppingListDto.setType(shoppingList.getType().name());

//        feShoppingListDto.setShoppingListItems(DtoFeMapper.mapToFeItemResponseDto(shoppingList.getShoppingListItems().stream().toList()));
        return feShoppingListDto;
    }

    public static ShoppingList mapToShoppingList(FeShoppingListRequestDto feShoppingListRequestDto) {
        ShoppingList shoppingList = new ShoppingList();

        shoppingList.setName(feShoppingListRequestDto.getName());
        shoppingList.setType(ShoppingListEnum.valueOf(feShoppingListRequestDto.getType()));

        return shoppingList;
    }
    //</editor-fold>

    //<editor-fold desc="FE --(add|modify)--> Cart">
    public static ShoppingListItem mapToShoppingListItem(FeItemRequestDto feItemRequestDto, Product product) {
        ShoppingListItem shoppingListItem = new ShoppingListItem();

        product.setOriginSku(feItemRequestDto.getOriginSku());

        shoppingListItem.setProduct(product);
        shoppingListItem.setQuantity(feItemRequestDto.getQuantity());
        shoppingListItem.setOriginItemId(feItemRequestDto.getOriginItemId()); // should be empty for AddToCart method (POST req)

        return shoppingListItem;
    }
    //</editor-fold>

    //<editor-fold desc="Cart --> FE">
    public static List<FeItemResponseDto> mapToFeItemResponseDto(List<ShoppingListItem> shoppingListItems) {
        return shoppingListItems.stream()
                .map(DtoFeMapper::mapToFeItemResponseDto)
                .collect(Collectors.toList());
    }

    public static FeItemResponseDto mapToFeItemResponseDto(ShoppingListItem shoppingListItem) {
        FeItemResponseDto feItemResponseDto = new FeItemResponseDto();

        feItemResponseDto.setOriginItemId(shoppingListItem.getOriginItemId());
        feItemResponseDto.setQuantity(shoppingListItem.getQuantity());
        feItemResponseDto.setPrice(shoppingListItem.getPrice());
        feItemResponseDto.setOriginSku(shoppingListItem.getProduct().getOriginSku());

        feItemResponseDto.setItemId(shoppingListItem.getId());
        feItemResponseDto.setCompany(shoppingListItem.getProduct().getCompany().getName().name());
        feItemResponseDto.setProduct(mapToFeProductResponseDto(shoppingListItem.getProduct()));

        return feItemResponseDto;
    }
    //</editor-fold>

    //<editor-fold desc="Products model --> FE">
    public static List<FeProductResponseDto> mapToFeProductResponseDto(List<Product> productDtos) {
        return productDtos.stream()
                .map(DtoFeMapper::mapToFeProductResponseDto)
                .collect(Collectors.toList());
    }

    public static FeProductResponseDto mapToFeProductResponseDto(Product product) {
        FeProductResponseDto feDto = new FeProductResponseDto();

        feDto.setCompany(product.getCompany().getName().name());

        feDto.setName(product.getName());
        feDto.setOriginProductId(product.getOriginProductId());
        feDto.setOriginSku(product.getOriginSku());

        feDto.setUnit(product.getUnit());
        feDto.setUnitStandardized(product.getUnitStandardized());
        feDto.setAmount(product.getAmount());

        feDto.setPriceFull(product.getPriceFull());
        feDto.setPriceCurrency(product.getPriceCurrency());
        feDto.setPricePerUnitFull(product.getPricePerUnitFull());
        feDto.setPricePerUnitCurrency(product.getPricePerUnitCurrency());
        feDto.setMaxAvailableAmount(product.getMaxAvailableAmount());

        feDto.setBaseUrl(product.getBaseUrl());
        feDto.setImgPath(product.getImgPath());
        feDto.setDescription(product.getDescription());

        return feDto;
    }
    //</editor-fold>

    //<editor-fold desc="FE --> Credential model">
    public static GroceryCredential mapToGroceryCredential(FeGroceryCredentialDto feGroceryCredentialDto, Company company) {
        GroceryCredential credential = new GroceryCredential();

        credential.setId(feGroceryCredentialDto.getId());
        credential.setUsername(feGroceryCredentialDto.getUsername());
        credential.setPassword(feGroceryCredentialDto.getPassword());
        credential.setCompany(company);

        return credential;
    }

    public static List<FeGroceryCredentialDto> mapToFeGroceryCredentialDto(List<GroceryCredential> groceryCredentials) {
        return groceryCredentials.stream()
                .map(DtoFeMapper::mapToFeGroceryCredentialDto)
                .collect(Collectors.toList());
    }

    public static FeGroceryCredentialDto mapToFeGroceryCredentialDto(GroceryCredential groceryCredential) {
        FeGroceryCredentialDto feGroceryCredentialDto = new FeGroceryCredentialDto();

        feGroceryCredentialDto.setId(groceryCredential.getId());
        feGroceryCredentialDto.setUsername(groceryCredential.getUsername());
        feGroceryCredentialDto.setPassword(groceryCredential.getPassword());
        feGroceryCredentialDto.setCompany(groceryCredential.getCompany().getName().name());

        return feGroceryCredentialDto;
    }
    //</editor-fold>
}
