package cz.gottlieb.groceryAggregator.dto.mapper;

import cz.gottlieb.groceryAggregator.dto.rohlik.*;
import cz.gottlieb.groceryAggregator.persistence.model.Company;
import cz.gottlieb.groceryAggregator.persistence.model.GroceryCredential;
import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingListItem;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

import static cz.gottlieb.groceryAggregator.configs.connectors.impl.RohlikConfigBaseImpl.serverBaseUrl;
import static cz.gottlieb.groceryAggregator.configs.connectors.impl.RohlikConfigBaseImpl.serverCdnUrl;

public class DtoRohlikMapper {

    //<editor-fold desc="FE --(add|modify)--> Cart">
    public static RohlikItemPostDto mapToRohlikCartItemPostDto(ShoppingListItem shoppingListItem) {
        RohlikItemPostDto rohlikItemPostDto = new RohlikItemPostDto();

        rohlikItemPostDto.setOriginProductId(Integer.parseInt(shoppingListItem.getProduct().getOriginSku()));
        rohlikItemPostDto.setQuantity(shoppingListItem.getQuantity());

        return rohlikItemPostDto;
    }

    public static RohlikItemPutDto mapToRohlikCartItemPutDto(ShoppingListItem shoppingListItem) {
        RohlikItemPutDto rohlikItemPutDto = new RohlikItemPutDto();

        rohlikItemPutDto.setOriginProductId(Integer.parseInt(shoppingListItem.getProduct().getOriginSku()));
        rohlikItemPutDto.setQuantity(shoppingListItem.getQuantity());

        return rohlikItemPutDto;
    }
    //</editor-fold>

    //<editor-fold desc="Cart --> FE">
    public static List<ShoppingListItem> mapToShoppingListItem(List<RohlikItemResponseDto> rohlikItemResponseDtos, Company company) {
        return rohlikItemResponseDtos.stream()
                .map(rohlikItemResDto -> mapToShoppingListItem(rohlikItemResDto, company))
                .collect(Collectors.toList());
    }

    public static ShoppingListItem mapToShoppingListItem(RohlikItemResponseDto rohlikItemResDto, Company company) {
        Product product = new Product();

        product.setCompany(company);

        product.setName(rohlikItemResDto.getProductName());
        product.setOriginProductId(String.valueOf(rohlikItemResDto.getProductId()));
        product.setOriginSku(String.valueOf(rohlikItemResDto.getProductId()));

        product.setUnit(rohlikItemResDto.getUnit());
        product.setUnitStandardized(rohlikItemResDto.getUnit());
        product.setAmount(rohlikItemResDto.getTextualAmount());

        product.setPriceFull(new BigDecimal(rohlikItemResDto.getPrice()));
        product.setPriceCurrency(rohlikItemResDto.getCurrency());
        product.setPricePerUnitFull(new BigDecimal(rohlikItemResDto.getOriginalPricePerUnit()));
        product.setPricePerUnitCurrency(rohlikItemResDto.getCurrency());
        product.setMaxAvailableAmount(rohlikItemResDto.getMaxBasketAmount());

        product.setBaseUrl(serverBaseUrl + "/" + rohlikItemResDto.getBaseLink());
        product.setImgPath(serverCdnUrl + rohlikItemResDto.getImgPath());
        product.setDescription("");
        product.setUpdateAt(LocalDateTime.now());

        ShoppingListItem shoppingListItem = new ShoppingListItem();

        shoppingListItem.setOriginItemId(String.valueOf(rohlikItemResDto.getOrderFieldId()));
        shoppingListItem.setQuantity(rohlikItemResDto.getQuantity());

        BigDecimal quantity = new BigDecimal(rohlikItemResDto.getQuantity());
        BigDecimal price = new BigDecimal(rohlikItemResDto.getPrice());
        shoppingListItem.setPrice(quantity.multiply(price));

        shoppingListItem.setProduct(product);

        return shoppingListItem;
    }
    //</editor-fold>

    //<editor-fold desc="Rohlik products --> Model">
    public static List<Product> mapToProduct(List<RohlikProductResponseDto> rohlikProductResponseDtos, Company company) {
        return rohlikProductResponseDtos.stream()
                .map(rohlikDto -> mapToProduct(rohlikDto, company))
                .collect(Collectors.toList());
    }

    public static Product mapToProduct(RohlikProductResponseDto rohlikDto, Company company) {
        Product product = new Product();

        product.setCompany(company);

        product.setName(rohlikDto.getProductName());
        product.setOriginProductId(String.valueOf(rohlikDto.getProductId()));
        product.setOriginSku(String.valueOf(rohlikDto.getProductId()));

        product.setUnit(rohlikDto.getUnit());
        product.setUnitStandardized(rohlikDto.getUnit());
        product.setAmount(rohlikDto.getTextualAmount());
        product.setMaxAvailableAmount(rohlikDto.getMaxBasketAmount());

        product.setPriceFull(new BigDecimal(rohlikDto.getPrice().getFull()));
        product.setPriceCurrency(rohlikDto.getPrice().getCurrency());

        product.setPricePerUnitFull(new BigDecimal(rohlikDto.getPricePerUnit().getFull()));
        product.setPricePerUnitCurrency(rohlikDto.getPricePerUnit().getCurrency());

        product.setBaseUrl(serverBaseUrl + "/" + rohlikDto.getBaseLink());
        product.setImgPath(serverCdnUrl + rohlikDto.getImgPath());
        product.setDescription("");
        product.setUpdateAt(LocalDateTime.now());

        return product;
    }
    //</editor-fold>

    //<editor-fold desc="Credential model --> Rohlik">
    public static RohlikCredentialDto mapToRohlikCredentialDto(GroceryCredential credential) {
        RohlikCredentialDto rohlikCredentialDto = new RohlikCredentialDto();

        rohlikCredentialDto.setPassword(credential.getPassword());
        rohlikCredentialDto.setEmail(credential.getUsername());

        return rohlikCredentialDto;
    }
    //</editor-fold>
}
