package cz.gottlieb.groceryAggregator.persistence.model;

import com.sun.istack.NotNull;
import lombok.*;
import org.hibernate.proxy.HibernateProxy;

import javax.persistence.*;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Objects;

@Getter
@Setter
@ToString
@RequiredArgsConstructor
@Entity
@Table(name = "Product", uniqueConstraints = {
        @UniqueConstraint(name = "uc_product_originproductid_companyid", columnNames = {"originProductId", "company_id"}),
        @UniqueConstraint(name = "uc_product_originsku_companyid", columnNames = {"originSku", "company_id"})
})
public class Product {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    private String name;

    private String originProductId;
    private String originSku; // used by 'findBy' repo method (saveOrUpdate, updateShoppingListItem)

    private String unit;
    private String unitStandardized;
    private String amount;

    private BigDecimal priceFull;
    private String priceCurrency;

    private BigDecimal pricePerUnitFull;
    private String pricePerUnitCurrency;

    private Integer maxAvailableAmount;

    private String baseUrl;
    private String imgPath;

    @Column(length = 4096)
    private String description;

    private LocalDateTime updateAt;

    @ManyToOne
    @JoinColumn(name = "company_id", nullable = false, foreignKey=@ForeignKey(name="fk_product_companyid"))
    private Company company;

/*
        obj.productName
        obj.textualAmount
        obj.price.full + " " + obj.price.currency + "/" + "ks"
        obj.pricePerUnit.full + " " + obj.pricePerUnit.currency + "/" + obj.unit
        obj.imgPath
        obj.deliveryRestriction
*/

    @Override
    public final boolean equals(Object o) {
        if (this == o) return true;
        if (o == null) return false;
        Class<?> oEffectiveClass = o instanceof HibernateProxy ? ((HibernateProxy) o).getHibernateLazyInitializer().getPersistentClass() : o.getClass();
        Class<?> thisEffectiveClass = this instanceof HibernateProxy ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass() : this.getClass();
        if (thisEffectiveClass != oEffectiveClass) return false;
        Product product = (Product) o;
        return getId() != null && Objects.equals(getId(), product.getId());
    }

    @Override
    public final int hashCode() {
        return this instanceof HibernateProxy ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass().hashCode() : getClass().hashCode();
    }
}
