package cz.gottlieb.groceryAggregator.persistence.model;

import com.sun.istack.NotNull;
import cz.gottlieb.groceryAggregator.utils.ShoppingListEnum;
import lombok.*;
import org.hibernate.proxy.HibernateProxy;
import org.springframework.validation.annotation.Validated;

import javax.persistence.*;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;
import java.util.*;

@Getter
@Setter
@ToString
@RequiredArgsConstructor
@Entity
public class ShoppingList {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotBlank
    private String name;

    @Enumerated(EnumType.STRING)
    @Column(nullable = false)
    private ShoppingListEnum type;

//    private BigDecimal totalPrice; // TODO not implemented yet => remove?

    @OneToMany
    @JoinColumn(name = "shopping_list_id", foreignKey = @ForeignKey(name = "fk_shoppinglistitem_shoppinglistid"))
    private Set<ShoppingListItem> shoppingListItems = new LinkedHashSet<>();

    @Override
    public final boolean equals(Object o) {
        if (this == o) return true;
        if (o == null) return false;
        Class<?> oEffectiveClass = o instanceof HibernateProxy ? ((HibernateProxy) o).getHibernateLazyInitializer().getPersistentClass() : o.getClass();
        Class<?> thisEffectiveClass = this instanceof HibernateProxy ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass() : this.getClass();
        if (thisEffectiveClass != oEffectiveClass) return false;
        ShoppingList that = (ShoppingList) o;
        return getId() != null && Objects.equals(getId(), that.getId());
    }

    @Override
    public final int hashCode() {
        return this instanceof HibernateProxy ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass().hashCode() : getClass().hashCode();
    }
}
