package cz.gottlieb.groceryAggregator.persistence.model;

import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.proxy.HibernateProxy;

import javax.persistence.*;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotEmpty;
import java.util.LinkedList;
import java.util.List;
import java.util.Objects;

@Getter
@Setter
@ToString
@RequiredArgsConstructor
@Entity
@Table(name = "User", uniqueConstraints = {
        @UniqueConstraint(name = "uc_user_email", columnNames = {"email"}),
        @UniqueConstraint(name = "uc_user_username", columnNames = {"username"})
})
public class User {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    private String ssoUuid; // Keycloak

    @Email(message = "Email is not valid", regexp = "^[a-zA-Z0-9_!#$%&'*+/=?`{|}~^.-]+@[a-zA-Z0-9.-]+$")
    @NotEmpty(message = "Email cannot be empty")
    private String email;

    @NotEmpty(message = "Username cannot be empty")
    private String username;

    private String firstName;
    private String lastName;

    private String passwordHash; // not used - Keycloak is used instead

    private String phoneNumber; // not used - Keycloak is not set up

    @OneToMany
    @JoinColumn(name = "user_id", foreignKey = @ForeignKey(name = "fk_grocerycredential_userid"))
    private List<GroceryCredential> credentials = new LinkedList<>();

    @OneToMany
    @JoinColumn(name = "user_id", foreignKey = @ForeignKey(name = "fk_shoppinglist_userid"))
    private List<ShoppingList> shoppingLists = new LinkedList<>();

    @OneToMany
    @JoinColumn(name = "user_id", foreignKey = @ForeignKey(name = "fk_productgroup_userid"))
    private List<ProductGroup> productGroups = new LinkedList<>();

    @OneToMany
    @JoinColumn(name = "user_id", foreignKey = @ForeignKey(name = "fk_watchdog_userid"))
    private List<WatchDog> watchDogs = new LinkedList<>();

    @Override
    public final boolean equals(Object o) {
        if (this == o) return true;
        if (o == null) return false;
        Class<?> oEffectiveClass = o instanceof HibernateProxy ? ((HibernateProxy) o).getHibernateLazyInitializer().getPersistentClass() : o.getClass();
        Class<?> thisEffectiveClass = this instanceof HibernateProxy ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass() : this.getClass();
        if (thisEffectiveClass != oEffectiveClass) return false;
        User user = (User) o;
        return getId() != null && Objects.equals(getId(), user.getId());
    }

    @Override
    public final int hashCode() {
        return this instanceof HibernateProxy ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass().hashCode() : getClass().hashCode();
    }
}
