package cz.gottlieb.groceryAggregator.services.connectors.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import cz.gottlieb.groceryAggregator.persistence.model.GroceryCredential;
import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.services.connectors.GroceryService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.util.Pair;
import org.springframework.http.HttpStatus;
import org.springframework.lang.Nullable;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.Objects;

@Slf4j
public abstract class GroceryBaseServiceImpl implements GroceryService {

    @Override
    public List<Product> findProductsByName(String name, @Nullable GroceryCredential credential) throws JsonProcessingException {
        log.info(">>> findProductsByName");

        Pair<HttpStatus, List<Product>> result;
        List<Product> productsResponse = List.of();

        if (credential != null) {
            HttpStatus loginStatus = login(credential);
            if (loginStatus.is2xxSuccessful()) {
                result = findProducts(name);
                productsResponse = result.getSecond();
            }
        } else {
            result = findProducts(name);
            productsResponse = result.getSecond();
        }
        return productsResponse;
    }

    @Override
    public Product findProductByNameAndSkuOrThrow(String name,
                                                  String originSku,
                                                  @Nullable GroceryCredential credential) {
        log.info(">>> findProductByNameAndSkuOrThrow");

        List<Product> productsResponse;

        try {
            productsResponse = findProductsByName(name, credential);
        } catch (JsonProcessingException e) {
            throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR, null);
        }

        Product newProduct = productsResponse.stream()
                .filter(prod -> Objects.equals(prod.getOriginSku(), originSku))
                .filter(prod -> Objects.equals(prod.getCompany().getName(), getType()))
                .findFirst()
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.NOT_FOUND,
                        "Product (name: " + name + " , sku: " + originSku + ") NOT FOUND online in " + getType().name()));

        return newProduct;
    }

}
