package cz.gottlieb.groceryAggregator.services.impl;

import cz.gottlieb.groceryAggregator.configs.connectors.GroceryConfig;
import cz.gottlieb.groceryAggregator.persistence.model.Company;
import cz.gottlieb.groceryAggregator.persistence.repository.CompanyRepository;
import cz.gottlieb.groceryAggregator.services.CompanyService;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.util.Optional;

@Service
public class CompanyServiceImpl implements CompanyService {

    private final CompanyRepository companyRepository;

    public CompanyServiceImpl(CompanyRepository companyRepository) {
        this.companyRepository = companyRepository;
    }

    @Override
    public Company findCompanyByNameOrThrow(String companyName) {
        Company company = companyRepository.findByName(GroceryEnum.valueOf(companyName))
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR,
                        "Company (" + companyName + ") NOT found"));
        return company;
    }

    @Override
    public Company setUpCompany(GroceryEnum groceryEnum, GroceryConfig config) {
        Optional<Company> foundCompany = companyRepository.findByName(groceryEnum);

        if (foundCompany.isPresent()) {
            return foundCompany.get();
        } else {
            Company company = new Company();

            company.setName(groceryEnum);
            company.setBaseUrl(config.getServerBaseUrl());
            company.setOrderUrl(config.getServerOrderUrl());
            company.setDeliveryPrice(config.getDeliveryPrice());
            company.setMinimalOrderPrice(config.getMinimalOrderPrice());
            company.setMinimalFreeDeliveryPrice(config.getMinimalFreeDeliveryPrice());

            return companyRepository.save(company);
        }
    }
}
