package cz.gottlieb.groceryAggregator.services.impl;

import cz.gottlieb.groceryAggregator.persistence.model.GroceryCredential;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.repository.GroceryCredentialRepository;
import cz.gottlieb.groceryAggregator.services.CredentialsService;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.util.Objects;
import java.util.Optional;

@Slf4j
@Service
public class CredentialsServiceImpl implements CredentialsService {

    private final GroceryCredentialRepository groceryCredentialRepository;

    public CredentialsServiceImpl(GroceryCredentialRepository groceryCredentialRepository) {
        this.groceryCredentialRepository = groceryCredentialRepository;
    }

    public GroceryCredential findCredentialsByGroceryNameOrThrow(User user, GroceryEnum grocery) {
        return user.getCredentials().stream()
                .filter(cred -> Objects.equals(cred.getCompany().getName(), grocery))
                .findFirst()
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.UNAUTHORIZED,
                        "GroceryCredential (" + grocery + ") NOT found")
                );
    }

    public Optional<GroceryCredential> findCredentialsByGroceryName(User user, GroceryEnum grocery) {
        return user.getCredentials().stream()
                .filter(cred -> Objects.equals(cred.getCompany().getName(), grocery))
                .findFirst();
    }

    public GroceryCredential findCredentialsByIdOrThrow(User user, Long id) {
        return user.getCredentials().stream()
                .filter(cred -> cred.getId().equals(id))
                .findFirst()
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.NOT_FOUND,
                        "GroceryCredential with id " + id + " not found"));
    }

    @Override
    public GroceryCredential save(GroceryCredential groceryCredential) {
        return groceryCredentialRepository.save(groceryCredential);
    }

    @Override
    public void deleteById(Long id) {
        groceryCredentialRepository.deleteById(id);
    }

}