package cz.gottlieb.groceryAggregator.services.impl;

import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingList;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingListItem;
import cz.gottlieb.groceryAggregator.persistence.repository.ProductRepository;
import cz.gottlieb.groceryAggregator.services.ProductService;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.util.Objects;
import java.util.Optional;

@Slf4j
@Service
public class ProductServiceImpl implements ProductService {


    private final ProductRepository productRepository;

    public ProductServiceImpl(ProductRepository productRepository) {
        this.productRepository = productRepository;
    }

    public Product findProductBySkuAndCompanyOrThrow(String originSku, String company) {
        return productRepository.findByOriginSkuAndCompany_Name(originSku, //  existingItem.getProduct().getOriginSku(); // TODO pointless?
                        GroceryEnum.valueOf(company))    //  existingItem.getProduct().getCompany().getName(); // TODO pointless?
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.BAD_REQUEST,    // TODO change error code (BAD_REQUEST ?)
                        "Product (originSku: " + originSku + ") NOT found"));
    }

    public Optional<Product> findProductDuplicityByItem(ShoppingList shoppingList, ShoppingListItem item) {
        return shoppingList.getShoppingListItems().stream()
                .map(ShoppingListItem::getProduct)
                .filter(prod -> Objects.equals(item.getProduct(), prod))
                .findFirst();
    }
}
