package cz.gottlieb.groceryAggregator.services.impl;

import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.repository.UserRepository;
import cz.gottlieb.groceryAggregator.services.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.util.Optional;

@Slf4j
@Service
public class UserServiceImpl implements UserService {

    private final UserRepository userRepository;

    public UserServiceImpl(UserRepository userRepository) {
        this.userRepository = userRepository;
    }

    public User getUserFromJwt(Jwt principal) {
        Optional<String> uuid = Optional.ofNullable((String) principal.getClaims().get("sub"));

        if (uuid.isEmpty()) {
            throw new ResponseStatusException(HttpStatus.FORBIDDEN,
                    "User CANNOT be authenticated.");
        }

        User user = userRepository.findBySsoUuid(uuid.get()).orElseGet(() -> {
            User newUser = new User();

            Optional<String> email = Optional.ofNullable((String) principal.getClaims().get("email"));
            Optional<String> username = Optional.ofNullable((String) principal.getClaims().get("preferred_username"));
            Optional<String> firstName = Optional.ofNullable((String) principal.getClaims().get("given_name"));
            Optional<String> lastName = Optional.ofNullable((String) principal.getClaims().get("family_name"));

            if (email.isEmpty() || username.isEmpty() || firstName.isEmpty() || lastName.isEmpty()) {
                // TODO is it possible? - depends on Keycloak settings
                throw new ResponseStatusException(HttpStatus.FORBIDDEN,
                        "User CANNOT be created because some user info is missing.");
            }

            newUser.setEmail(email.get());
            newUser.setSsoUuid(uuid.get());
            newUser.setUsername(username.get());
            newUser.setFirstName(firstName.get());
            newUser.setLastName(lastName.get());

            userRepository.save(newUser);
            return newUser;
        });

        return user;
    }

    public User findUserByEmailOrThrow(String email, String methodName) {
        //  TODO delete?
        //  return ResponseEntity.notFound().build();
        //  return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
        //  throw new ResponseStatusException(HttpStatus.UNAUTHORIZED, "");
        String errorMsg = "User not found";

        return userRepository.findByEmail(email)
                .orElseThrow(() -> {
                    log.info("method={}, userEmail={}, message={}", methodName, email, errorMsg);
                    return new ResponseStatusException(HttpStatus.UNAUTHORIZED, errorMsg);
                });
    }

    public void save(User user) {
        userRepository.save(user);
    }
}
