import {NgModule} from '@angular/core';
import {RouterModule, Routes} from '@angular/router';
import {DashboardComponent} from "./components/dashboard/dashboard.component";
import {CartsComponent} from "./components/cart/carts/carts.component";
import {PageNotFoundComponent} from "./components/shared/page-not-found/page-not-found.component";
import {GroceryCredentialComponent} from "./components/credential/grocery-credential/grocery-credential.component";
import {WatchDogComponent} from "./components/dog/watch-dog/watch-dog.component";
import {ShoppingListsComponent} from "./components/shoppingList/shopping-lists/shopping-lists.component";
import {
  ShoppingListItemsComponent
} from "./components/shoppingList/detail/shopping-list-items/shopping-list-items.component";
import {AuthorizationGuard} from "./guard/AuthorizationGuard";
import {LoginPageComponent} from "./components/login-page/login-page.component";

const routes: Routes = [
  {path: '', redirectTo: '/dashboard', pathMatch: 'full'},
  {path: 'login', component: LoginPageComponent},
  {path: 'dashboard', component: DashboardComponent, canActivate: [AuthorizationGuard]},
  {path: 'carts', component: CartsComponent, canActivate: [AuthorizationGuard]},
  {path: 'shopping-lists', component: ShoppingListsComponent, canActivate: [AuthorizationGuard]},
  {path: 'shopping-lists/:listId/detail', component: ShoppingListItemsComponent, canActivate: [AuthorizationGuard]},
  {path: 'watchdogs', component: WatchDogComponent, canActivate: [AuthorizationGuard]},
  {path: 'credentials', component: GroceryCredentialComponent, canActivate: [AuthorizationGuard]},
  {path: '**', component: PageNotFoundComponent},  // Wildcard route for a 404 page
];

@NgModule({
  imports: [RouterModule.forRoot(routes)],
  exports: [RouterModule]
})
export class AppRoutingModule {
}
