import {BreakpointObserver} from '@angular/cdk/layout';
import {Component, ViewChild} from '@angular/core';
import {MatSidenav} from '@angular/material/sidenav';
import {OidcSecurityService} from "angular-auth-oidc-client";
import {UserService} from "./services/rest/user.service";

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent {
  @ViewChild(MatSidenav)
  sidenav!: MatSidenav;
  isMobile = true;
  isCollapsed = true;

  isLoggedIn: boolean = false;
  userData: Object = {};
  userName: string = "";

  constructor(private observer: BreakpointObserver,
              private oidcSecurityService: OidcSecurityService,
              private userService: UserService) {
  }

  ngOnInit() {
    this.observer.observe(['(max-width: 800px)']).subscribe((screenSize) => {
      if (screenSize.matches) {
        this.isMobile = true;
      } else {
        this.isMobile = false;
      }
    });

    this.oidcSecurityService.checkAuth().subscribe(({isAuthenticated, userData}) => {
      this.isLoggedIn = isAuthenticated;
      this.userData = userData;

      console.log("userData"); // TODO delete it
      console.log(userData); // TODO delete it

      if (userData?.name) {
        this.userName = userData.name;
      }
    });

  }

  login() {
    this.oidcSecurityService.authorize();
  }

  logout() {
    this.userService.logout().subscribe(value =>
      console.log(value)
    );

    this.oidcSecurityService.logoff().subscribe(result => {
      console.log(result);
    });
  }

  toggleMenu() {
    if (this.isMobile) {
      this.sidenav.toggle();
      this.isCollapsed = false;
    } else {
      this.sidenav.open();
      this.isCollapsed = !this.isCollapsed;
    }
  }
}
