import {InjectionToken, NgModule} from '@angular/core';
import {BrowserModule} from '@angular/platform-browser';

import {AppRoutingModule} from './app-routing.module';
import {AppComponent} from './app.component';
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';
import {MatToolbarModule} from "@angular/material/toolbar";
import {MatIconModule} from "@angular/material/icon";
import {MatSidenavModule} from "@angular/material/sidenav";
import {MatListModule} from "@angular/material/list";
import {MatButtonModule} from "@angular/material/button";
import {MatGridListModule} from "@angular/material/grid-list";
import {DashboardComponent} from './components/dashboard/dashboard.component';
import {CartsComponent} from './components/cart/carts/carts.component';
import {PageNotFoundComponent} from './components/shared/page-not-found/page-not-found.component';
import {MatRippleModule} from "@angular/material/core";
import {MatCardModule} from "@angular/material/card";
import {MatMenuModule} from "@angular/material/menu";
import {GroceryCredentialComponent} from './components/credential/grocery-credential/grocery-credential.component';
import {MatTableModule} from "@angular/material/table";
import {HTTP_INTERCEPTORS, HttpClient, HttpClientModule} from "@angular/common/http";
import {MatFormFieldModule} from "@angular/material/form-field";
import {FormsModule, ReactiveFormsModule} from "@angular/forms";
import {MatInputModule} from "@angular/material/input";
import {MatSelectModule} from "@angular/material/select";
import {
  GroceryCredentialDetailComponent
} from './components/credential/grocery-credential-detail/grocery-credential-detail.component';
import {WatchDogComponent} from './components/dog/watch-dog/watch-dog.component';
import {WatchDogDetailComponent} from './components/dog/watch-dog-detail/watch-dog-detail.component';
import {MatAutocompleteModule} from "@angular/material/autocomplete";
import {NgOptimizedImage} from "@angular/common";
import {ProductSearchComponent} from './components/shared/product-search/product-search.component';
import {SpinnerComponent} from './components/shared/spinner/spinner.component';
import {ShoppingListsComponent} from './components/shoppingList/shopping-lists/shopping-lists.component';
import {GroceryRestService} from "./services/rest/grocery-rest.service";
import {FuncsService} from "./services/funcs.service";
import {RohlikComponent} from './components/cart/rohlik/rohlik.component';
import {BillaComponent} from './components/cart/billa/billa.component';
import {CartItemDetailComponent} from './components/cart/cart-item-detail/cart-item-detail.component';
import {SessionInterceptor} from "./interceptor/session.interceptor";
import {
  DialogNewShoppingListComponent
} from './components/cart/dialog-new-shopping-list/dialog-new-shopping-list.component';
import {MatDialogActions, MatDialogContent, MatDialogModule} from "@angular/material/dialog";
import {
  ShoppingListsDetailComponent
} from './components/shoppingList/shopping-lists-detail/shopping-lists-detail.component';
import {
  ShoppingListItemsComponent
} from './components/shoppingList/detail/shopping-list-items/shopping-list-items.component';
import {AuthInterceptor, AuthModule, LogLevel} from "angular-auth-oidc-client";
import {LoginPageComponent} from './components/login-page/login-page.component';
import {PassedInitialConfig} from "angular-auth-oidc-client/lib/auth-config";
import {LoadingInterceptor} from "./interceptor/loading.interceptor";

export const rohlikRest = new InjectionToken<GroceryRestService>('rohlikGroceryRestService');
export const billaRest = new InjectionToken<GroceryRestService>('billaGroceryRestService');

export function groceryRestServiceFactory(url: string) {
  return (http: HttpClient, funcsService: FuncsService) =>
    new GroceryRestService(http, funcsService, url);
}

const oAuth2Conf: PassedInitialConfig = {
  config: {
    authority: 'http://localhost:8180/realms/myrealm',
    redirectUrl: window.location.origin,
    postLogoutRedirectUri: window.location.origin,
    clientId: 'myclient',
    scope: 'openid offline_access read write',
    responseType: 'code',
    silentRenew: true,
    useRefreshToken: true,
    logLevel: LogLevel.Debug,
    ignoreNonceAfterRefresh: true,
    secureRoutes: ['http://localhost:8180', 'http://localhost:8081'],
  },
};

@NgModule({
  declarations: [
    AppComponent,
    DashboardComponent,
    CartsComponent,
    PageNotFoundComponent,
    GroceryCredentialComponent,
    GroceryCredentialDetailComponent,
    WatchDogComponent,
    WatchDogDetailComponent,
    ProductSearchComponent,
    SpinnerComponent,
    ShoppingListsComponent,
    RohlikComponent,
    BillaComponent,
    CartItemDetailComponent,
    DialogNewShoppingListComponent,
    ShoppingListsDetailComponent,
    ShoppingListItemsComponent,
    LoginPageComponent,
  ],
  imports: [
    AuthModule.forRoot(oAuth2Conf),
    HttpClientModule,
    BrowserModule,
    AppRoutingModule,
    BrowserAnimationsModule,
    MatIconModule,
    MatButtonModule,
    MatToolbarModule,
    MatSidenavModule,
    MatListModule,
    MatGridListModule,
    MatRippleModule,
    MatCardModule,
    MatMenuModule,
    MatTableModule,
    MatFormFieldModule,
    MatInputModule,
    FormsModule,
    MatSelectModule,
    ReactiveFormsModule,
    MatAutocompleteModule,
    NgOptimizedImage,
    MatDialogModule,
    MatDialogContent,
    MatDialogActions,
  ],
  providers: [
    {
      provide: HTTP_INTERCEPTORS,
      useClass: SessionInterceptor,
      multi: true
    },
    {
      provide: HTTP_INTERCEPTORS,
      useClass: LoadingInterceptor,
      multi: true
    },
    {
      provide: HTTP_INTERCEPTORS,
      useClass: AuthInterceptor,
      multi: true
    },
    {
      provide: rohlikRest,
      useFactory: groceryRestServiceFactory("rohlik"),
      deps: [HttpClient, FuncsService]
    },
    {
      provide: billaRest,
      useFactory: groceryRestServiceFactory("billa"),
      deps: [HttpClient, FuncsService],
    },
  ],
  bootstrap: [AppComponent]
})
export class AppModule {
}
