import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {ItemDto} from "../../../dto/ItemDto";
import {GroceryRestService} from "../../../services/rest/grocery-rest.service";
import {HttpClient} from "@angular/common/http";
import {FuncsService} from "../../../services/funcs.service";
import {ItemInputTypeEnum} from "../../../util/ItemInputTypeEnum";
import {ActivatedRoute} from "@angular/router";
import {ShoppingListItemService} from "../../../services/rest/shopping-list-item.service";
import {MessageService} from "../../../services/message.service";

@Component({
  selector: 'app-cart-item-detail',
  templateUrl: './cart-item-detail.component.html',
  styleUrl: './cart-item-detail.component.scss'
})
export class CartItemDetailComponent implements OnInit {
  protected readonly Math = Math;
  protected readonly ItemInputTypeEnum = ItemInputTypeEnum;

  @Input({required: true})
  item!: ItemDto;

  @Input()
  inputType: ItemInputTypeEnum = ItemInputTypeEnum.CART;

  @Output()
  itemChange = new EventEmitter<ItemDto>();

  listId?: number;
  restService?: GroceryRestService;

  constructor(private http: HttpClient,
              private funcsService: FuncsService,
              private route: ActivatedRoute,
              private itemService: ShoppingListItemService,
              private messageService: MessageService) {
  }

  ngOnInit(): void {
    if (this.inputType === ItemInputTypeEnum.SHOPPING_LIST) {
      this.listId = parseInt(this.route.snapshot.paramMap.get('listId') ?? '');
    }

    this.restService = new GroceryRestService(
      this.http,
      this.funcsService,
      this.item.company.toString().toLowerCase()
    );
  }

  public increaseItem() {
    console.log("increaseItem");
    this.item.quantity++;

    console.log(this.item);
    this.updateItem();
  }

  public decreaseItem() {
    console.log("decreaseItem");
    if (--this.item.quantity === 0) {
      return this.deleteItem();
    }

    console.log(this.item);
    this.updateItem();
  }

  public deleteItem() {
    console.log("deleteItem");
    this.item.quantity = 0;

    console.log(this.item);
    this.updateWithDelete();
  }

  private updateItem() {
    if (this.inputType === ItemInputTypeEnum.SHOPPING_LIST && this.listId) {
      this.itemService.updateShoppingListItem(this.listId, this.item).subscribe(item => {
        this.item = item;
        this.itemChange.emit(this.item);
      });
    } else {
      this.restService?.updateCartItem(this.item).subscribe(item => {
        this.item = item;
        this.itemChange.emit(this.item);
      });
    }
  }

  private updateWithDelete() {
    if (this.inputType === ItemInputTypeEnum.SHOPPING_LIST && this.listId && this.item.itemId) {
      this.itemService.deleteShoppingListItem(this.listId, this.item.itemId).subscribe({
        complete: () => {
          this.funcsService.redirectTo();
        }
      });
    } else {
      this.restService?.updateCartItem(this.item).subscribe({
        complete: () => {
          this.funcsService.redirectTo();
        }
      });
    }
  }

  findAlt() {
    if(!this.item.product){
      return;
    }

    console.log(this.item.product.name);

    this.messageService.setProductAltRequest(this.item.product);
  }
}
