import {Component, OnInit} from '@angular/core';
import {MatDialog} from "@angular/material/dialog";
import {DialogNewShoppingListComponent} from "../dialog-new-shopping-list/dialog-new-shopping-list.component";
import {Observable} from "rxjs";
import {FuncsService} from "../../../services/funcs.service";
import {ShoppingListService} from "../../../services/rest/shopping-list.service";
import {ShoppingListDto} from "../../../dto/ShoppingListDto";
import {ShoppingListEnum} from "../../../util/ShoppingListEnum";
import {GroceryCredentialService} from "../../../services/rest/grocery-credential-service.service";
import {GroceryCredentialDto} from "../../../dto/GroceryCredentialDto";
import {CompanyEnum} from "../../../util/CompanyEnum";

@Component({
  selector: 'app-carts',
  templateUrl: './carts.component.html',
  styleUrl: './carts.component.scss'
})
export class CartsComponent implements OnInit {

  credentialDtos: GroceryCredentialDto[] | undefined;

  rohlikCredential: GroceryCredentialDto | undefined;
  billaCredential: GroceryCredentialDto | undefined;

  constructor(private dialog: MatDialog,
              private shoppingListService: ShoppingListService,
              private funcsService: FuncsService,
              private credentialService: GroceryCredentialService) {
  }

  ngOnInit(): void {
    this.loadAllCredentials();
  }

  private loadAllCredentials(): void {
    this.credentialService.getAllCredentials().subscribe(credentials => {
      this.credentialDtos = credentials;

      credentials.forEach(value => {
        if (value.company === CompanyEnum.ROHLIK) {
          this.rohlikCredential = value;
        }
        if (value.company === CompanyEnum.BILLA) {
          this.billaCredential = value;
        }
      });

    });
  }

  saveAsNewList() {
    let dialogRef = this.dialog.open(DialogNewShoppingListComponent, {
      enterAnimationDuration: '150ms',
      exitAnimationDuration: '150ms'
    });

    dialogRef.afterClosed().subscribe(userChoiceName => {
      if (typeof userChoiceName !== 'string') {
        return;
      }

      this.createNewShoppingList(userChoiceName).subscribe(shoppingList => {
        if (!shoppingList.id) {
          return; // ERROR // TODO notify error
        }

        this.shoppingListService.copyCartsToShoppingList(shoppingList.id).subscribe(result => {
          this.funcsService.redirectTo(); // TODO inform user and redirect to new shopping list?
        });
      });

    });
  }

  private createNewShoppingList(name: string): Observable<ShoppingListDto> {
    console.log("try to create new ShoppingList with name: " + name);

    let shoppingList = {
      name: name,
      type: ShoppingListEnum.WISHLIST,
    } as ShoppingListDto;

    return this.shoppingListService.addShoppingList(shoppingList);
  }
}
