import {Component, Inject, Input, OnDestroy, OnInit} from '@angular/core';
import {rohlikRest} from "../../../app.module";
import {GroceryRestService} from "../../../services/rest/grocery-rest.service";
import {ItemDto} from "../../../dto/ItemDto";
import {MessageService} from "../../../services/message.service";
import {FuncsService} from "../../../services/funcs.service";
import {takeWhile} from "rxjs";
import {Constants} from "../../../util/Constants";
import {ItemInputTypeEnum} from "../../../util/ItemInputTypeEnum";
import {ShoppingListItemService} from "../../../services/rest/shopping-list-item.service";
import {ActivatedRoute} from "@angular/router";
import {AggregatedSearchService} from "../../../services/rest/aggregated-search.service";

@Component({
  selector: 'app-rohlik',
  templateUrl: './rohlik.component.html',
  styleUrl: './rohlik.component.scss'
})
export class RohlikComponent implements OnInit, OnDestroy {
  protected readonly ItemInputTypeEnum = ItemInputTypeEnum;
  alive: boolean = true;
  URL_CART_REVIEW = Constants.URL_CART_REVIEW_ROHLIK;
  listId?: number;

  @Input()
  inputType: ItemInputTypeEnum = ItemInputTypeEnum.CART;

  rohlikItems: ItemDto[] = [];

  constructor(@Inject(rohlikRest) private rohlikRestService: GroceryRestService,
              private messageService: MessageService,
              private funcsService: FuncsService,
              private route: ActivatedRoute,
              private itemService: ShoppingListItemService,
              private searchService: AggregatedSearchService) {
  }

  ngOnInit(): void {
    if (this.inputType === ItemInputTypeEnum.SHOPPING_LIST) {
      this.listId = parseInt(this.route.snapshot.paramMap.get('listId') ?? '');
      this.loadShoppingList();
    } else {
      this.loadCart();
    }
    this.bindToSelectedProduct();
  }

  private loadShoppingList() {
    this.messageService.getRohlikItems().pipe(takeWhile(() => this.alive))
      .subscribe(items => {
        return this.rohlikItems = items;
      });
  }

  private loadCart() {
    this.rohlikRestService.getAllCartItems().subscribe(items => {
      return this.rohlikItems = items;
    });
  }

  ngOnDestroy() {
    this.alive = false;
  }

  private bindToSelectedProduct(): void {
    this.messageService.getSelectedRohlikProduct().pipe(takeWhile(() => this.alive))
      .subscribe(product => {
        console.log("catching product:"); // TODO delete it
        console.log(product); // TODO delete it

        let itemTmp: ItemDto = {
          company: product.company,
          originSku: product.originSku,
          quantity: 1
        } as ItemDto;

        if (this.inputType === ItemInputTypeEnum.SHOPPING_LIST && this.listId) {
          this.searchService.findProductByNameAndSku(product.company, product.originSku, product.name)
            .subscribe(value => {
              console.log(value); // TODO delete it

              this.itemService.addShoppingListItem(this.listId!, itemTmp).subscribe(item => {
                console.log("added item:"); // TODO delete it
                console.log(item); // TODO delete it
                this.funcsService.redirectTo();
              });
            });
        } else {
          this.rohlikRestService.addCartItem(itemTmp).subscribe(item => {
            console.log("added item:"); // TODO delete it
            console.log(item); // TODO delete it
            this.funcsService.redirectTo();
          });
        }
      });
  }

  getSum() {
    if (this.rohlikItems === null || this.rohlikItems === undefined || this.rohlikItems.length === 0) {
      return 0;
    }
    let sum = this.rohlikItems.map(value => value.price ?? 0)
      .reduce((accumulator, currentValue) => accumulator + currentValue);
    return sum.toLocaleString(undefined, {maximumFractionDigits: 2, minimumFractionDigits: 0});
  }

  clearCart() {
    if (!confirm("Are you sure, you want to clear the cart?")) {
      return;
    }

    if (this.inputType === ItemInputTypeEnum.SHOPPING_LIST && this.listId) {
      this.rohlikItems.filter(i => i.itemId !== null)
        .forEach(item =>
          this.itemService.deleteShoppingListItem(this.listId!, item.itemId!).subscribe({
            complete: () => this.funcsService.redirectTo()
          }));
    } else {
      this.rohlikRestService.deleteAllCartItems().subscribe({
        complete: () => this.funcsService.redirectTo()
      });
    }
  }
}
