import {Component, OnInit} from '@angular/core';
import {ShoppingListDto} from "../../../../dto/ShoppingListDto";
import {ShoppingListService} from "../../../../services/rest/shopping-list.service";
import {ActivatedRoute} from "@angular/router";
import {ItemDto} from "../../../../dto/ItemDto";
import {ShoppingListItemService} from "../../../../services/rest/shopping-list-item.service";
import {MessageService} from "../../../../services/message.service";
import {ItemInputTypeEnum} from "../../../../util/ItemInputTypeEnum";
import {FuncsService} from "../../../../services/funcs.service";

@Component({
  selector: 'app-shopping-list-items',
  templateUrl: './shopping-list-items.component.html',
  styleUrl: './shopping-list-items.component.scss'
})
export class ShoppingListItemsComponent implements OnInit {
  readonly ItemInputTypeEnum = ItemInputTypeEnum;
  shoppingList?: ShoppingListDto;
  shoppingListItem?: ItemDto[];

  constructor(private route: ActivatedRoute,
              private shoppingListService: ShoppingListService,
              private shoppingListItemService: ShoppingListItemService,
              private messageService: MessageService,
              private funcsService: FuncsService) {
  }

  ngOnInit(): void {
    const id = parseInt(this.route.snapshot.paramMap.get('listId') ?? '');
    if (id) {
      this.shoppingListService.getShoppingListById(id).subscribe(dto => {
        this.shoppingList = dto;
        console.log("this.shoppingList")
        console.log(this.shoppingList);
      });

      this.shoppingListItemService.getAllShoppingListItems(id).subscribe(dto => {
        this.shoppingListItem = dto;

        console.log("this.shoppingListItem");
        console.log(this.shoppingListItem);

        this.messageService.setItems(this.shoppingListItem);
      });
    }
  }

  load() {
    if (!confirm("Are you sure, you want to load this list into your carts?")) {
      return;
    }
    const shoppingListId = this.shoppingList?.id; // TODO
    if (!shoppingListId) {
      return;
    }

    this.shoppingListService.copyShoppingListToCarts(shoppingListId).subscribe({
      complete: () => this.funcsService.redirectTo("/carts")
    });
  }
}
