import {Injectable} from '@angular/core';
import {Observable, Subject} from "rxjs";
import {ProductDto} from "../dto/ProductDto";
import {CompanyEnum} from "../util/CompanyEnum";
import {ItemDto} from "../dto/ItemDto";

@Injectable({
  providedIn: 'root'
})
export class MessageService {
  private productAltRequest$ = new Subject<ProductDto>();

  private selectedProduct$ = new Subject<ProductDto>();
  private selectedRohlikProduct$ = new Subject<ProductDto>();
  private selectedBillaProduct$ = new Subject<ProductDto>();

  private rohlikItemArray$ = new Subject<ItemDto[]>();
  private billaItemArray$ = new Subject<ItemDto[]>();

  constructor() {
  }

  getProductAltRequest(): Observable<ProductDto> {
    return this.productAltRequest$.asObservable();
  }

  setProductAltRequest(product: ProductDto): void {
    this.productAltRequest$.next(product);
  }

  getSelectedProduct(): Observable<ProductDto> {
    return this.selectedProduct$.asObservable();
  }

  getSelectedRohlikProduct(): Observable<ProductDto> {
    return this.selectedRohlikProduct$.asObservable();
  }

  getSelectedBillaProduct(): Observable<ProductDto> {
    return this.selectedBillaProduct$.asObservable();
  }

  setSelectedProduct(product: ProductDto): void {
    this.selectedProduct$.next(product);

    if (product.company === CompanyEnum.ROHLIK) {
      this.selectedRohlikProduct$.next(product);
    } else if (product.company === CompanyEnum.BILLA) {
      this.selectedBillaProduct$.next(product);
    }
  }

  setItems(items: ItemDto[]): void {
    console.log(">>> setItems"); // TODO delete it
    console.log(items); // TODO delete it

    this.filterItemsByCompanyAndSend(items, CompanyEnum.ROHLIK, this.rohlikItemArray$);
    this.filterItemsByCompanyAndSend(items, CompanyEnum.BILLA, this.billaItemArray$);
  }

  getRohlikItems(): Observable<ItemDto[]> {
    return this.rohlikItemArray$.asObservable();
  }

  getBillaItems(): Observable<ItemDto[]> {
    return this.billaItemArray$.asObservable();
  }

  private filterItemsByCompanyAndSend(items: ItemDto[], companyEnum: CompanyEnum, itemArray$: Subject<ItemDto[]>) {
    const itemArray = items.filter(value =>
      value.company === companyEnum);

    if (itemArray.length > 0) {
      itemArray$.next(itemArray);
    }
  }
}
