import {Injectable} from '@angular/core';
import {catchError, Observable, tap} from "rxjs";
import {ProductDto} from "../../dto/ProductDto";
import {Constants} from "../../util/Constants";
import {HttpClient, HttpHeaders, HttpParams} from "@angular/common/http";
import {FuncsService} from "../funcs.service";

@Injectable({
  providedIn: 'root'
})
export class AggregatedSearchService {
  private url: string = Constants.SERVER_URL + '/grocery';

  httpOptions = {
    headers: new HttpHeaders({'Content-Type': 'application/json'})
  };

  constructor(private http: HttpClient,
              private funcsService: FuncsService) {
  }

  /** GET findProductsByName from the server */
  findProductsByName(name: string, filteredCompany?: string): Observable<ProductDto[]> {
    const url = `${this.url}/findByName`;

    let params = new HttpParams();
    params = params.append("name", name);

    if (filteredCompany) {
      params = params.append("filteredCompany", filteredCompany);
    }

    return this.http.get<ProductDto[]>(url, {params: params})
      .pipe(
        tap(_ => this.funcsService.log('fetched product')),
        catchError(this.funcsService.handleError<ProductDto[]>('findProductsByName', []))
      );
  }

  /** POST findProductsByNameAndSku from the server */
  findProductByNameAndSku(company: string, sku: string, name: string): Observable<ProductDto> {
    const url = `${this.url}/findByNameAndSku/save`;

    let params = new HttpParams();
    params = params.append("company", company);
    params = params.append("originSku", sku);
    params = params.append("name", name);

    let options = {
      ...this.httpOptions,
      params: params
    }

    return this.http.post<ProductDto>(url, null, options)
      .pipe(
        tap(_ => this.funcsService.log('fetched product')),
        catchError(this.funcsService.handleError<ProductDto>('findProductByNameAndSku'))
      );
  }

}
